from ROOT import *

# Testing
# ======================
outfileName = 'c2030.root'
tfin = TFile.Open(outfileName, 'READ')

# Helper function to fill histograms
def fill_hists(tr, tag):
  nevents = tr.GetEntries()

  nbins, xmin, xmax=25, -1, 1
#   hname="BDT_S_{0}".format(tag)
#   h1 = TH1F(hname, hname, nbins, xmin, xmax)
#   h1.Sumw2()
  hname="BDTG_S_{0}".format(tag)
  h2 = TH1F(hname, hname, nbins, xmin, xmax)
  h2.Sumw2()
#   hname="BDT_B_{0}".format(tag)
#   h3 = TH1F(hname, hname, nbins, xmin, xmax)
#   h3.Sumw2()
  hname="BDTG_B_{0}".format(tag)
  h4 = TH1F(hname, hname, nbins, xmin, xmax)
  h4.Sumw2()

  nbins, xmin, xmax=25, 0, 1
  hname="MLP_S_{0}".format(tag)
  h5 = TH1F(hname, hname, nbins, xmin, xmax)
  h5.Sumw2()
  hname="MLP_B_{0}".format(tag)
  h6 = TH1F(hname, hname, nbins, xmin, xmax)
  h6.Sumw2()
  
  for i in range(nevents):
    tr.GetEntry(i)
 
    # BDT = tr.BDT     
    BDTG = tr.BDTG     
    MLP = tr.MLP_ANN
    # Signal
    if tr.classID == 0:
    #   h1.Fill(BDT)
      h2.Fill(BDTG)
      h5.Fill(MLP)
    # Background
    if tr.classID == 1:
    #   h3.Fill(BDT)
      h4.Fill(BDTG)
      h6.Fill(MLP)
    
  return [h2, h4, h5, h6]

# Helper function to normalize hists
def norm_hists(h):

  h_new = h.Clone()
  hname = h.GetName() + "_normalized"
  h_new.SetName(hname)
  h_new.SetTitle(hname)
  ntot = h.Integral()
  if ntot!=0:
    h_new.Scale(1./ntot)

  return h_new

# Fill histograms
# gROOT.LoadMacro("AtlasStyle.C")
# SetAtlasStyle()
gROOT.SetStyle("ATLAS")

tr_train = tfin.Get("DAOD/TrainTree")
h_train = fill_hists(tr_train, "train")

tr_test = tfin.Get("DAOD/TestTree")
h_test = fill_hists(tr_test, "test")

outFileName = '../results/BDTG_TrainTest_2v3.root'
outFile = TFile.Open(outFileName, 'RECREATE')
h_train[0].Write()
h_train[1].Write()
h_test[0].Scale(2)
h_test[1].Scale(2)
h_test[0].Write()
h_test[1].Write()
outFile.Close()

outFileName = '../results/MLP_TrainTest_2v3.root'
outFile = TFile.Open(outFileName, 'RECREATE')
h_train[2].Write()
h_train[3].Write()
h_test[2].Scale(2)
h_test[3].Scale(2)
h_test[2].Write()
h_test[3].Write()
outFile.Close()

# Plotting
myc = TCanvas("c", "c", 800, 600)
myc.SetFillColor(0)
myc.cd()

# Train vs Test
# ===============
# BDTG
h1 = norm_hists(h_train[0])
h1.GetXaxis().SetTitle("BDTG")
h1.GetYaxis().SetTitle("Norm.Events")
h1.SetMarkerColor(46)
# h1.SetMarkerSize(3)
h1.Draw()

h2 = norm_hists(h_test[0])
h2.SetLineColor(46)
h2.SetLineWidth(4)
h2.Draw("same hist")

h3 = norm_hists(h_train[1])
h3.SetMarkerColor(38)
# h3.SetMarkerSize(3)
h3.Draw("same")

h4 = norm_hists(h_test[1])
h4.SetLineColor(38)
h4.SetLineWidth(4)
h4.Draw("same hist")

ymin = 0
ymax = max(h1.GetMaximum(), h2.GetMaximum(), h3.GetMaximum(), h4.GetMaximum())
h1.GetYaxis().SetRangeUser(ymin, ymax*1.2)

# Draw legends
lIy = 0.90
lg = TLegend(0.70, lIy-0.25, 0.90, lIy)
lg.SetBorderSize(0)
lg.SetFillStyle(0)
lg.SetTextFont(42)
lg.SetTextSize(0.04)
lg.AddEntry(h1, "Training S", "lp")
lg.AddEntry(h2, "Testing S", "l")
lg.AddEntry(h3, "Training B", "lp")
lg.AddEntry(h4, "Testing B", "l")
lg.Draw()

myc.Draw()
myc.SaveAs("../results/BDTG_TrainTest_2v3.png")


# # BDTG
# h1 = norm_hists(h_train[1])
# h1.GetXaxis().SetTitle("BDTG")
# h1.GetYaxis().SetTitle("A.U.")
# h1.Draw()
# h2 = norm_hists(h_test[1])
# h2.Draw("same hist")
# h3 = norm_hists(h_train[3])
# h3.SetLineColor(2)
# h3.SetMarkerColor(2)
# h3.Draw("same")
# h4 = norm_hists(h_test[3])
# h4.SetLineColor(2)
# h4.SetMarkerColor(2)
# h4.Draw("same hist")

# ymin = 0
# ymax = max(h1.GetMaximum(), h2.GetMaximum(), h3.GetMaximum(), h4.GetMaximum())
# h1.GetYaxis().SetRangeUser(ymin, ymax*1.2)

# # Draw legends
# lIy = 0.92
# lg = TLegend(0.60, lIy-0.25, 0.85, lIy)
# lg.SetBorderSize(0)
# lg.SetFillStyle(0)
# lg.SetTextFont(42)
# lg.SetTextSize(0.04)
# lg.AddEntry(h1, "Training S", "lp")
# lg.AddEntry(h2, "Testing S", "l")
# lg.AddEntry(h3, "Training B", "lp")
# lg.AddEntry(h4, "Testing B", "l")
# lg.Draw()

# myc.Draw()
# myc.SaveAs("BDT_cla_test_2.png")

# ANN MLP
h1 = norm_hists(h_train[2])
h1.GetXaxis().SetTitle("MLP")
h1.GetYaxis().SetTitle("Norm.Events")
h1.SetMarkerColor(46)
# h1.SetMarkerSize(3)
h1.Draw()

h2 = norm_hists(h_test[2])
h2.SetLineColor(46)
h2.SetLineWidth(4)
h2.Draw("same hist")

h3 = norm_hists(h_train[3])
h3.SetMarkerColor(38)
# h3.SetMarkerSize(3)
h3.Draw("same")

h4 = norm_hists(h_test[3])
h4.SetLineColor(38)
h4.SetLineWidth(4)
h4.Draw("same hist")

ymin = 0
ymax = max(h1.GetMaximum(), h2.GetMaximum(), h3.GetMaximum(), h4.GetMaximum())
h1.GetYaxis().SetRangeUser(ymin, ymax*1.2)

# Draw legends
lIy = 0.90
lg = TLegend(0.70, lIy-0.25, 0.90, lIy)
lg.SetBorderSize(0)
lg.SetFillStyle(0)
lg.SetTextFont(42)
lg.SetTextSize(0.04)
lg.AddEntry(h1, "Training S", "lp")
lg.AddEntry(h2, "Testing S", "l")
lg.AddEntry(h3, "Training B", "lp")
lg.AddEntry(h4, "Testing B", "l")
lg.Draw()

myc.Draw()
myc.SaveAs("../results/MLP_TrainTest_2v3.png")
